@@ -2,7 +2,7 @@
  * fsp-session.c
  *
  * Copyright (C) 2010-2012 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU Lesser General
@@ -68,6 +68,9 @@ struct _FspSessionPrivate
   gchar *token;
   gchar *token_secret;
 
+  gchar *tmp_token;
+  gchar *tmp_token_secret;
+
   gboolean using_gnome_proxy;
   SoupURI *proxy_uri;
   SoupSession *soup_session;
@@ -106,6 +109,11 @@ typedef struct
   gulong               cancellable_id;
 } GCancellableData;
 
+typedef enum {
+  TOKEN_TYPE_PERMANENT,
+  TOKEN_TYPE_TEMPORARY,
+} TokenType;
+
 typedef enum {
   AUTHORIZATION_METHOD_ORIGINAL,
   AUTHORIZATION_METHOD_OAUTH_1
@@ -241,11 +249,12 @@ static gchar *
 _get_signed_url                       (FspSession          *self,
                                        const gchar         *url,
                                        AuthorizationMethod  auth_method,
+                                       TokenType            token_type,
                                        const gchar         *first_param,
                                        ... );
 
 static void
-_clear_token_information              (FspSession *self);
+_clear_temporary_token                (FspSession *self);
 
 static void
 _perform_async_request                  (SoupSession         *soup_session,
@@ -425,6 +434,8 @@ fsp_session_finalize                    (GObject* object)
   g_free (self->priv->secret);
   g_free (self->priv->token);
   g_free (self->priv->token_secret);
+  g_free (self->priv->tmp_token);
+  g_free (self->priv->tmp_token_secret);
 
   /* Call superclass */
   G_OBJECT_CLASS (fsp_session_parent_class)->finalize(object);
@@ -481,6 +492,8 @@ fsp_session_init                        (FspSession *self)
   self->priv->secret = NULL;
   self->priv->token = NULL;
   self->priv->token_secret = NULL;
+  self->priv->tmp_token = NULL;
+  self->priv->tmp_token_secret = NULL;
 
   self->priv->using_gnome_proxy = FALSE;
   self->priv->proxy_uri = NULL;
@@ -1184,6 +1197,7 @@ static gchar *
 _get_signed_url                       (FspSession          *self,
                                        const gchar         *url,
                                        AuthorizationMethod  auth_method,
+                                       TokenType            token_type,
                                        const gchar         *first_param,
                                        ... )
 {
@@ -1194,6 +1208,8 @@ _get_signed_url                       (FspSession          *self,
   gchar *signed_query = NULL;
   gchar *api_sig = NULL;
   gchar *retval = NULL;
+  const gchar *token = NULL;
+  const gchar *token_secret = NULL;
 
   g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
   g_return_val_if_fail (url != NULL, NULL);
@@ -1202,20 +1218,31 @@ _get_signed_url                       (FspSession          *self,
   priv = self->priv;
   va_start (args, first_param);
 
+  if (token_type == TOKEN_TYPE_PERMANENT)
+    {
+      token = priv->token;
+      token_secret = priv->token_secret;
+    }
+  else
+    {
+      token = priv->tmp_token;
+      token_secret = priv->tmp_token_secret;
+    }
+
   /* Get the hash table for the params */
   table = _get_params_table_from_valist (first_param, args);
 
   /* Fill the table with mandatory parameters */
   if (auth_method == AUTHORIZATION_METHOD_OAUTH_1)
     {
-      _fill_hash_table_with_oauth_params (table, priv->api_key, priv->token);
-      signing_key = g_strdup_printf ("%s&%s", priv->secret, priv->token_secret ? priv->token_secret : "");
+      _fill_hash_table_with_oauth_params (table, priv->api_key, token);
+      signing_key = g_strdup_printf ("%s&%s", priv->secret, token_secret ? token_secret : "");
     }
   else
     {
       g_hash_table_insert (table, g_strdup ("api_key"), g_strdup (priv->api_key));
-      if (priv->token)
-        g_hash_table_insert (table, g_strdup ("auth_token"), g_strdup (priv->token));
+      if (token)
+        g_hash_table_insert (table, g_strdup ("auth_token"), g_strdup (token));
       signing_key = g_strdup (priv->secret);
     }
 
@@ -1238,23 +1265,23 @@ _get_signed_url                       (FspSession          *self,
 }
 
 static void
-_clear_token_information              (FspSession *self)
+_clear_temporary_token                (FspSession *self)
 {
   FspSessionPrivate *priv = NULL;
 
   g_return_if_fail (FSP_IS_SESSION (self));
 
   priv = self->priv;
-  if (priv->token)
+  if (priv->tmp_token)
     {
-      g_free (priv->token);
-      priv->token = NULL;
+      g_free (priv->tmp_token);
+      priv->tmp_token = NULL;
     }
 
-  if (priv->token_secret)
+  if (priv->tmp_token_secret)
     {
-      g_free (priv->token_secret);
-      priv->token_secret = NULL; 
+      g_free (priv->tmp_token_secret);
+      priv->tmp_token_secret = NULL;
     }
 }
 
@@ -1284,7 +1311,7 @@ _perform_async_request                  (SoupSession         *soup_session,
   clos->object = source_object;
   clos->soup_session = soup_session;
   clos->soup_message = msg;
-  clos->cancellable = cancellable;
+  clos->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
   clos->callback = callback;
   clos->source_tag = source_tag;
   clos->data = data;
@@ -1395,7 +1422,7 @@ _build_async_result_and_complete        (AsyncRequestData *clos,
   /* Make sure the "cancelled" signal gets disconnected in another
      iteration of the main loop to avoid a dead-lock with itself */
   cancellable_data = g_slice_new0 (GCancellableData);
-  cancellable_data->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+  cancellable_data->cancellable = cancellable;
   cancellable_data->cancellable_id = cancellable_id;
 
   g_idle_add ((GSourceFunc) _disconnect_cancellable_on_idle, cancellable_data);
@@ -1689,6 +1716,7 @@ fsp_session_set_http_proxy              (FspSession *self,
           soup_uri_set_port (proxy_uri, actual_port);
           soup_uri_set_user (proxy_uri, actual_user);
           soup_uri_set_password (proxy_uri, actual_password);
+          soup_uri_set_path (proxy_uri, "");
         }
     }
 
@@ -1776,7 +1804,7 @@ fsp_session_set_token_secret            (FspSession  *self,
 /* Get authorization URL */
 
 void
-fsp_session_get_auth_url_async          (FspSession          *self,
+fsp_session_get_auth_url                (FspSession          *self,
                                          GCancellable        *c,
                                          GAsyncReadyCallback  cb,
                                          gpointer             data)
@@ -1785,20 +1813,21 @@ fsp_session_get_auth_url_async          (FspSession          *self,
 
   g_return_if_fail (FSP_IS_SESSION (self));
 
-  /* If we're here, we're no longer interested in former tokens */
-  _clear_token_information (self);
+  /* Make sure there's no previous temporary data */
+  _clear_temporary_token (self);
 
   /* Build the signed url */
   url = _get_signed_url (self,
                          FLICKR_REQUEST_TOKEN_OAUTH_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_TEMPORARY,
                          "oauth_callback", OAUTH_CALLBACK_URL,
                          NULL);
 
   /* Perform the async request */
   _perform_async_request (self->priv->soup_session, url,
                           _get_request_token_session_cb, G_OBJECT (self),
-                          c, cb, fsp_session_get_auth_url_async, data);
+                          c, cb, fsp_session_get_auth_url, data);
 
   g_free (url);
 }
@@ -1816,14 +1845,14 @@ fsp_session_get_auth_url_finish         (FspSession    *self,
 
   auth_token =
     FSP_DATA_AUTH_TOKEN (_finish_async_request (G_OBJECT (self), res,
-                                                fsp_session_get_auth_url_async,
+                                                fsp_session_get_auth_url,
                                                 error));
   /* Build the auth URL from the request token */
   if (auth_token != NULL)
     {
-      /* Save the data */
-      fsp_session_set_token (self, auth_token->token);
-      fsp_session_set_token_secret (self, auth_token->token_secret);
+      /* Save the temporaty data */
+      self->priv->tmp_token = g_strdup (auth_token->token);
+      self->priv->tmp_token_secret = g_strdup (auth_token->token_secret);
 
       /* Build the authorization url */
       auth_url = g_strdup_printf ("http://www.flickr.com/services/oauth/authorize"
@@ -1838,7 +1867,7 @@ fsp_session_get_auth_url_finish         (FspSession    *self,
 /* Complete authorization */
 
 void
-fsp_session_complete_auth_async         (FspSession          *self,
+fsp_session_complete_auth               (FspSession          *self,
                                          const gchar         *code,
                                          GCancellable        *c,
                                          GAsyncReadyCallback  cb,
@@ -1850,7 +1879,7 @@ fsp_session_complete_auth_async         (FspSession          *self,
   g_return_if_fail (cb != NULL);
 
   priv = self->priv;
-  if (priv->token != NULL && priv->token_secret != NULL)
+  if (priv->tmp_token != NULL && priv->tmp_token_secret != NULL)
     {
       gchar *url = NULL;
 
@@ -1858,13 +1887,14 @@ fsp_session_complete_auth_async         (FspSession          *self,
       url = _get_signed_url (self,
                              FLICKR_ACCESS_TOKEN_OAUTH_URL,
                              AUTHORIZATION_METHOD_OAUTH_1,
+                             TOKEN_TYPE_TEMPORARY,
                              "oauth_verifier", code,
                              NULL);
 
       /* Perform the async request */
       _perform_async_request (priv->soup_session, url,
                               _get_access_token_soup_session_cb, G_OBJECT (self),
-                              c, cb, fsp_session_complete_auth_async, data);
+                              c, cb, fsp_session_complete_auth, data);
 
       g_free (url);
     }
@@ -1876,8 +1906,6 @@ fsp_session_complete_auth_async         (FspSession          *self,
       err = g_error_new (FSP_ERROR, FSP_ERROR_OAUTH_NOT_AUTHORIZED_YET, "Not authorized yet");
       g_simple_async_report_gerror_in_idle (G_OBJECT (self),
                                             cb, data, err);
-      /* Ensure we clean things up */
-      _clear_token_information (self);
     }
 }
 
@@ -1893,7 +1921,7 @@ fsp_session_complete_auth_finish        (FspSession    *self,
 
   auth_token =
     FSP_DATA_AUTH_TOKEN (_finish_async_request (G_OBJECT (self), res,
-                                                fsp_session_complete_auth_async,
+                                                fsp_session_complete_auth,
                                                 error));
 
   /* Complete the authorization saving the token if present */
@@ -1903,11 +1931,14 @@ fsp_session_complete_auth_finish        (FspSession    *self,
       fsp_session_set_token_secret (self, auth_token->token_secret);
     }
 
+  /* Ensure we clean things up */
+  _clear_temporary_token (self);
+
   return auth_token;
 }
 
 void
-fsp_session_exchange_token_async        (FspSession          *self,
+fsp_session_exchange_token              (FspSession          *self,
                                          GCancellable        *c,
                                          GAsyncReadyCallback cb,
                                          gpointer             data)
@@ -1926,13 +1957,14 @@ fsp_session_exchange_token_async        (FspSession          *self,
       url = _get_signed_url (self,
                              FLICKR_API_BASE_URL,
                              AUTHORIZATION_METHOD_ORIGINAL,
+                             TOKEN_TYPE_PERMANENT,
                              "method", "flickr.auth.oauth.getAccessToken",
                              NULL);
 
       /* Perform the async request */
       _perform_async_request (priv->soup_session, url,
                               _exchange_token_soup_session_cb, G_OBJECT (self),
-                              c, cb, fsp_session_exchange_token_async, data);
+                              c, cb, fsp_session_exchange_token, data);
 
       g_free (url);
     }
@@ -1944,8 +1976,6 @@ fsp_session_exchange_token_async        (FspSession          *self,
       err = g_error_new (FSP_ERROR, FSP_ERROR_OAUTH_NOT_AUTHORIZED_YET, "Not authorized yet");
       g_simple_async_report_gerror_in_idle (G_OBJECT (self),
                                             cb, data, err);
-      /* Ensure we clean things up */
-      _clear_token_information (self);
     }
 }
 
@@ -1961,7 +1991,7 @@ fsp_session_exchange_token_finish       (FspSession    *self,
 
   auth_token =
     FSP_DATA_AUTH_TOKEN (_finish_async_request (G_OBJECT (self), res,
-                                                fsp_session_exchange_token_async,
+                                                fsp_session_exchange_token,
                                                 error));
   if (auth_token != NULL)
     {
@@ -1972,7 +2002,7 @@ fsp_session_exchange_token_finish       (FspSession    *self,
 }
 
 void
-fsp_session_check_auth_info_async       (FspSession          *self,
+fsp_session_check_auth_info             (FspSession          *self,
                                          GCancellable        *c,
                                          GAsyncReadyCallback  cb,
                                          gpointer             data)
@@ -1991,13 +2021,14 @@ fsp_session_check_auth_info_async       (FspSession          *self,
       url = _get_signed_url (self,
                              FLICKR_API_BASE_URL,
                              AUTHORIZATION_METHOD_OAUTH_1,
+                             TOKEN_TYPE_PERMANENT,
                              "method", "flickr.auth.oauth.checkToken",
                              NULL);
 
       /* Perform the async request */
       _perform_async_request (priv->soup_session, url,
                               _check_token_soup_session_cb, G_OBJECT (self),
-                              c, cb, fsp_session_check_auth_info_async, data);
+                              c, cb, fsp_session_check_auth_info, data);
 
       g_free (url);
     }
@@ -2024,13 +2055,13 @@ fsp_session_check_auth_info_finish      (FspSession    *self,
 
   auth_token =
     FSP_DATA_AUTH_TOKEN (_finish_async_request (G_OBJECT (self), res,
-                                                fsp_session_check_auth_info_async,
+                                                fsp_session_check_auth_info,
                                                 error));
   return auth_token;
 }
 
 void
-fsp_session_get_upload_status_async     (FspSession          *self,
+fsp_session_get_upload_status           (FspSession          *self,
                                          GCancellable        *c,
                                          GAsyncReadyCallback cb,
                                          gpointer             data)
@@ -2049,13 +2080,14 @@ fsp_session_get_upload_status_async     (FspSession          *self,
       url = _get_signed_url (self,
                              FLICKR_API_BASE_URL,
                              AUTHORIZATION_METHOD_OAUTH_1,
+                             TOKEN_TYPE_PERMANENT,
                              "method", "flickr.people.getUploadStatus",
                              NULL);
 
       /* Perform the async request */
       _perform_async_request (priv->soup_session, url,
                               _get_upload_status_soup_session_cb, G_OBJECT (self),
-                              c, cb, fsp_session_get_upload_status_async, data);
+                              c, cb, fsp_session_get_upload_status, data);
 
       g_free (url);
     }
@@ -2082,13 +2114,13 @@ fsp_session_get_upload_status_finish    (FspSession    *self,
 
   upload_status =
     FSP_DATA_UPLOAD_STATUS (_finish_async_request (G_OBJECT (self), res,
-                                                   fsp_session_get_upload_status_async,
+                                                   fsp_session_get_upload_status,
                                                    error));
   return upload_status;
 }
 
 void
-fsp_session_upload_async                (FspSession          *self,
+fsp_session_upload                      (FspSession          *self,
                                          const gchar         *fileuri,
                                          const gchar         *title,
                                          const gchar         *description,
@@ -2151,9 +2183,9 @@ fsp_session_upload_async                (FspSession          *self,
   ard_clos->object = G_OBJECT (self);
   ard_clos->soup_session = soup_session;
   ard_clos->soup_message = NULL;
-  ard_clos->cancellable = cancellable;
+  ard_clos->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
   ard_clos->callback = callback;
-  ard_clos->source_tag = fsp_session_upload_async;
+  ard_clos->source_tag = fsp_session_upload;
   ard_clos->data = data;
 
   /* Save important data for the upload process itself */
@@ -2175,12 +2207,12 @@ fsp_session_upload_finish               (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
   return (gchar*) _finish_async_request (G_OBJECT (self), res,
-                                         fsp_session_upload_async,
+                                         fsp_session_upload,
                                          error);
 }
 
 void
-fsp_session_get_info_async              (FspSession          *self,
+fsp_session_get_info                    (FspSession          *self,
                                          const gchar         *photo_id,
                                          GCancellable        *cancellable,
                                          GAsyncReadyCallback  callback,
@@ -2196,6 +2228,7 @@ fsp_session_get_info_async              (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.photos.getInfo",
                          "photo_id", photo_id,
                          NULL);
@@ -2204,7 +2237,7 @@ fsp_session_get_info_async              (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _photo_get_info_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_get_info_async, data);
+                          cancellable, callback, fsp_session_get_info, data);
 
   g_free (url);
 }
@@ -2221,13 +2254,13 @@ fsp_session_get_info_finish             (FspSession    *self,
 
   photo_info =
     FSP_DATA_PHOTO_INFO (_finish_async_request (G_OBJECT (self), res,
-                                                fsp_session_get_info_async,
+                                                fsp_session_get_info,
                                                 error));
   return photo_info;
 }
 
 void
-fsp_session_get_photosets_async         (FspSession          *self,
+fsp_session_get_photosets               (FspSession          *self,
                                          GCancellable        *cancellable,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
@@ -2241,6 +2274,7 @@ fsp_session_get_photosets_async         (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.photosets.getList",
                          NULL);
 
@@ -2248,7 +2282,7 @@ fsp_session_get_photosets_async         (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _get_photosets_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_get_photosets_async, data);
+                          cancellable, callback, fsp_session_get_photosets, data);
 
   g_free (url);
 }
@@ -2262,12 +2296,12 @@ fsp_session_get_photosets_finish        (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
   return (GSList*) _finish_async_request (G_OBJECT (self), res,
-                                          fsp_session_get_photosets_async,
+                                          fsp_session_get_photosets,
                                           error);
 }
 
 void
-fsp_session_add_to_photoset_async       (FspSession          *self,
+fsp_session_add_to_photoset             (FspSession          *self,
                                          const gchar         *photo_id,
                                          const gchar         *photoset_id,
                                          GCancellable        *cancellable,
@@ -2285,6 +2319,7 @@ fsp_session_add_to_photoset_async       (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.photosets.addPhoto",
                          "photo_id", photo_id,
                          "photoset_id", photoset_id,
@@ -2294,7 +2329,7 @@ fsp_session_add_to_photoset_async       (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _add_to_photoset_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_add_to_photoset_async, data);
+                          cancellable, callback, fsp_session_add_to_photoset, data);
 
   g_free (url);
 }
@@ -2310,13 +2345,13 @@ fsp_session_add_to_photoset_finish      (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
   result = _finish_async_request (G_OBJECT (self), res,
-                                  fsp_session_add_to_photoset_async, error);
+                                  fsp_session_add_to_photoset, error);
 
   return result ? TRUE : FALSE;
 }
 
 void
-fsp_session_create_photoset_async       (FspSession          *self,
+fsp_session_create_photoset             (FspSession          *self,
                                          const gchar         *title,
                                          const gchar         *description,
                                          const gchar         *primary_photo_id,
@@ -2335,6 +2370,7 @@ fsp_session_create_photoset_async       (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.photosets.create",
                          "title", title,
                          "description", description ? description : "",
@@ -2345,7 +2381,7 @@ fsp_session_create_photoset_async       (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _create_photoset_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_create_photoset_async, data);
+                          cancellable, callback, fsp_session_create_photoset, data);
 
   g_free (url);
 }
@@ -2362,13 +2398,13 @@ fsp_session_create_photoset_finish      (FspSession    *self,
 
   photoset_id =
     (gchar*) _finish_async_request (G_OBJECT (self), res,
-                                    fsp_session_create_photoset_async,
+                                    fsp_session_create_photoset,
                                     error);
   return photoset_id;
 }
 
 void
-fsp_session_get_groups_async            (FspSession          *self,
+fsp_session_get_groups                  (FspSession          *self,
                                          GCancellable        *cancellable,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
@@ -2382,6 +2418,7 @@ fsp_session_get_groups_async            (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.groups.pools.getGroups",
                          NULL);
 
@@ -2389,7 +2426,7 @@ fsp_session_get_groups_async            (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _get_groups_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_get_groups_async, data);
+                          cancellable, callback, fsp_session_get_groups, data);
 
   g_free (url);
 }
@@ -2403,12 +2440,12 @@ fsp_session_get_groups_finish           (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
   return (GSList*) _finish_async_request (G_OBJECT (self), res,
-                                          fsp_session_get_groups_async,
+                                          fsp_session_get_groups,
                                           error);
 }
 
 void
-fsp_session_add_to_group_async          (FspSession          *self,
+fsp_session_add_to_group                (FspSession          *self,
                                          const gchar         *photo_id,
                                          const gchar         *group_id,
                                          GCancellable        *cancellable,
@@ -2426,6 +2463,7 @@ fsp_session_add_to_group_async          (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.groups.pools.add",
                          "photo_id", photo_id,
                          "group_id", group_id,
@@ -2435,7 +2473,7 @@ fsp_session_add_to_group_async          (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _add_to_group_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_add_to_group_async, data);
+                          cancellable, callback, fsp_session_add_to_group, data);
 
   g_free (url);
 }
@@ -2451,13 +2489,13 @@ fsp_session_add_to_group_finish         (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
   result = _finish_async_request (G_OBJECT (self), res,
-                                  fsp_session_add_to_group_async, error);
+                                  fsp_session_add_to_group, error);
 
   return result ? TRUE : FALSE;
 }
 
 void
-fsp_session_get_tags_list_async         (FspSession          *self,
+fsp_session_get_tags_list               (FspSession          *self,
                                          GCancellable        *cancellable,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
@@ -2471,6 +2509,7 @@ fsp_session_get_tags_list_async         (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.tags.getListUser",
                          NULL);
 
@@ -2478,7 +2517,7 @@ fsp_session_get_tags_list_async         (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _get_tags_list_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_get_tags_list_async, data);
+                          cancellable, callback, fsp_session_get_tags_list, data);
 
   g_free (url);
 }
@@ -2492,11 +2531,11 @@ fsp_session_get_tags_list_finish        (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
   return (GSList*) _finish_async_request (G_OBJECT (self), res,
-                                          fsp_session_get_tags_list_async, error);
+                                          fsp_session_get_tags_list, error);
 }
 
 void
-fsp_session_set_license_async           (FspSession          *self,
+fsp_session_set_license                 (FspSession          *self,
                                          const gchar         *photo_id,
                                          FspLicense          license,
                                          GCancellable        *cancellable,
@@ -2515,6 +2554,7 @@ fsp_session_set_license_async           (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.photos.licenses.setLicense",
                          "photo_id", photo_id,
                          "license_id", license_str,
@@ -2525,7 +2565,7 @@ fsp_session_set_license_async           (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _set_license_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_set_license_async, data);
+                          cancellable, callback, fsp_session_set_license, data);
 
   g_free (url);
 }
@@ -2541,13 +2581,13 @@ fsp_session_set_license_finish          (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
   result = _finish_async_request (G_OBJECT (self), res,
-                                  fsp_session_set_license_async, error);
+                                  fsp_session_set_license, error);
 
   return result ? TRUE : FALSE;
 }
 
 void
-fsp_session_set_location_async           (FspSession          *self,
+fsp_session_set_location                 (FspSession          *self,
                                           const gchar         *photo_id,
                                           FspDataLocation     *location,
                                           GCancellable        *cancellable,
@@ -2577,6 +2617,7 @@ fsp_session_set_location_async           (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.photos.geo.setLocation",
                          "photo_id", photo_id,
                          "lat", lat_str,
@@ -2591,7 +2632,7 @@ fsp_session_set_location_async           (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _set_location_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_set_location_async, data);
+                          cancellable, callback, fsp_session_set_location, data);
 
   g_free (url);
 }
@@ -2607,13 +2648,13 @@ fsp_session_set_location_finish          (FspSession    *self,
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
   result = _finish_async_request (G_OBJECT (self), res,
-                                  fsp_session_set_location_async, error);
+                                  fsp_session_set_location, error);
 
   return result ? TRUE : FALSE;
 }
 
 void
-fsp_session_get_location_async           (FspSession          *self,
+fsp_session_get_location                 (FspSession          *self,
                                           const gchar         *photo_id,
                                           GCancellable        *cancellable,
                                           GAsyncReadyCallback  callback,
@@ -2628,6 +2669,7 @@ fsp_session_get_location_async           (FspSession          *self,
   url = _get_signed_url (self,
                          FLICKR_API_BASE_URL,
                          AUTHORIZATION_METHOD_OAUTH_1,
+                         TOKEN_TYPE_PERMANENT,
                          "method", "flickr.photos.geo.getLocation",
                          "photo_id", photo_id,
                          NULL);
@@ -2636,7 +2678,7 @@ fsp_session_get_location_async           (FspSession          *self,
   soup_session = _get_soup_session (self);
   _perform_async_request (soup_session, url,
                           _get_location_soup_session_cb, G_OBJECT (self),
-                          cancellable, callback, fsp_session_get_location_async, data);
+                          cancellable, callback, fsp_session_get_location, data);
 
   g_free (url);
 }
@@ -2653,7 +2695,7 @@ fsp_session_get_location_finish          (FspSession    *self,
 
   location =
     FSP_DATA_LOCATION (_finish_async_request (G_OBJECT (self), res,
-                                              fsp_session_get_location_async,
+                                              fsp_session_get_location,
                                               error));
   return location;
 }
