@@ -2,7 +2,7 @@
  * frogr-add-tags-dialog.c -- Frogr 'add tags' dialog
  *
  * Copyright (C) 2009-2012 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -25,7 +25,7 @@
 #include "frogr-global-defs.h"
 #include "frogr-live-entry.h"
 #include "frogr-main-view.h"
-#include "frogr-main-view-model.h"
+#include "frogr-model.h"
 #include "frogr-picture.h"
 
 #include <config.h>
@@ -52,14 +52,24 @@ enum  {
 
 /* Prototypes */
 
+static void _set_pictures (FrogrAddTagsDialog *self, const GSList *pictures);
+
 static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data);
 
 /* Private API */
 
+static void
+_set_pictures (FrogrAddTagsDialog *self, const GSList *pictures)
+{
+  FrogrAddTagsDialogPrivate *priv = FROGR_ADD_TAGS_DIALOG_GET_PRIVATE (self);
+  priv->pictures = g_slist_copy ((GSList*) pictures);
+  g_slist_foreach (priv->pictures, (GFunc)g_object_ref, NULL);
+}
+
 static void
 _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
 {
-  if (response == GTK_RESPONSE_OK)
+  if (response == GTK_RESPONSE_ACCEPT)
     {
       FrogrAddTagsDialog *self = NULL;
       FrogrAddTagsDialogPrivate *priv = NULL;
@@ -75,8 +85,7 @@ _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
       /* Check if there's something to add */
       if (tags && !g_str_equal (tags, ""))
         {
-          FrogrMainView *mainview = NULL;
-          FrogrMainViewModel *model = NULL;
+          FrogrModel *model = NULL;
           FrogrPicture *picture = NULL;
           GSList *item = NULL;
 
@@ -90,9 +99,8 @@ _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
             }
 
           /* Add tags to the model */
-          mainview = frogr_controller_get_main_view (frogr_controller_get_instance ());
-          model = frogr_main_view_get_model (mainview);
-          frogr_main_view_model_add_local_tags_from_string (model, tags);
+          model = frogr_controller_get_model (frogr_controller_get_instance ());
+          frogr_model_add_local_tags_from_string (model, tags);
         }
 
       /* Free */
@@ -109,12 +117,10 @@ _frogr_add_tags_dialog_set_property (GObject *object,
                                      const GValue *value,
                                      GParamSpec *pspec)
 {
-  FrogrAddTagsDialogPrivate *priv = FROGR_ADD_TAGS_DIALOG_GET_PRIVATE (object);
-
   switch (prop_id)
     {
     case PROP_PICTURES:
-      priv->pictures = (GSList *) g_value_get_pointer (value);
+      _set_pictures (FROGR_ADD_TAGS_DIALOG (object), g_value_get_pointer (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -189,10 +195,10 @@ frogr_add_tags_dialog_init (FrogrAddTagsDialog *self)
 
   /* Create widgets */
   gtk_dialog_add_buttons (GTK_DIALOG (self),
-                          GTK_STOCK_OK,
-                          GTK_RESPONSE_OK,
                           GTK_STOCK_CANCEL,
                           GTK_RESPONSE_CANCEL,
+                          GTK_STOCK_ADD,
+                          GTK_RESPONSE_ACCEPT,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
 
@@ -215,7 +221,7 @@ frogr_add_tags_dialog_init (FrogrAddTagsDialog *self)
 /* Public API */
 
 void
-frogr_add_tags_dialog_show (GtkWindow *parent, GSList *pictures, GSList *tags)
+frogr_add_tags_dialog_show (GtkWindow *parent, const GSList *pictures, const GSList *tags)
 {
   FrogrConfig *config = NULL;
   GObject *new = NULL;
