@@ -1,8 +1,8 @@
 /*
  * frogr-add-to-group-dialog.c -- 'Add to group' dialog
  *
- * Copyright (C) 2010-2011 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Copyright (C) 2010-2012 Mario Sanchez Prada
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -20,7 +20,9 @@
 
 #include "frogr-add-to-group-dialog.h"
 
+#include "frogr-controller.h"
 #include "frogr-group.h"
+#include "frogr-model.h"
 #include "frogr-picture.h"
 
 #include <config.h>
@@ -68,6 +70,10 @@ enum {
 
 /* Prototypes */
 
+static void _set_pictures (FrogrAddToGroupDialog *self, const GSList *pictures);
+
+static void _set_groups (FrogrAddToGroupDialog *self, const GSList *groups);
+
 static GtkWidget *_create_tree_view (FrogrAddToGroupDialog *self);
 
 static void _column_clicked_cb (GtkTreeViewColumn *col, gpointer data);
@@ -97,6 +103,23 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
 
 /* Private API */
 
+static void
+_set_pictures (FrogrAddToGroupDialog *self, const GSList *pictures)
+{
+  FrogrAddToGroupDialogPrivate *priv = FROGR_ADD_TO_GROUP_DIALOG_GET_PRIVATE (self);
+  priv->pictures = g_slist_copy ((GSList*) pictures);
+  g_slist_foreach (priv->pictures, (GFunc)g_object_ref, NULL);
+}
+
+static void _set_groups (FrogrAddToGroupDialog *self, const GSList *groups)
+{
+  FrogrAddToGroupDialogPrivate *priv = NULL;
+
+  priv = FROGR_ADD_TO_GROUP_DIALOG_GET_PRIVATE (self);
+  priv->groups = g_slist_copy ((GSList*)groups);
+  g_slist_foreach (priv->groups, (GFunc)g_object_ref, NULL);
+}
+
 static GtkWidget *
 _create_tree_view (FrogrAddToGroupDialog *self)
 {
@@ -376,10 +399,21 @@ _update_pictures (FrogrAddToGroupDialog *self)
   priv = FROGR_ADD_TO_GROUP_DIALOG_GET_PRIVATE (self);
 
   selected_groups = _get_selected_groups (self);
-  for (item = priv->pictures; item; item = g_slist_next (item))
+  if (selected_groups)
     {
-      picture = FROGR_PICTURE (item->data);
-      frogr_picture_set_groups (picture, selected_groups);
+      FrogrModel *model = NULL;
+
+      for (item = priv->pictures; item; item = g_slist_next (item))
+        {
+          picture = FROGR_PICTURE (item->data);
+
+          /* frogr_picture_set_groups expects transfer-full for groups */
+          g_slist_foreach (selected_groups, (GFunc) g_object_ref, NULL);
+          frogr_picture_set_groups (picture, g_slist_copy (selected_groups));
+        }
+
+      model = frogr_controller_get_model (frogr_controller_get_instance ());
+      frogr_model_notify_changes_in_pictures (model);
     }
   g_slist_free (selected_groups);
 }
@@ -387,7 +421,7 @@ _update_pictures (FrogrAddToGroupDialog *self)
 static void
 _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
 {
-  if (response == GTK_RESPONSE_OK)
+  if (response == GTK_RESPONSE_ACCEPT)
     {
       FrogrAddToGroupDialog *self = NULL;
 
@@ -405,15 +439,13 @@ _frogr_add_to_group_dialog_set_property (GObject *object,
                                          const GValue *value,
                                          GParamSpec *pspec)
 {
-  FrogrAddToGroupDialogPrivate *priv = FROGR_ADD_TO_GROUP_DIALOG_GET_PRIVATE (object);
-
   switch (prop_id)
     {
     case PROP_PICTURES:
-      priv->pictures = (GSList *) g_value_get_pointer (value);
+      _set_pictures (FROGR_ADD_TO_GROUP_DIALOG (object), g_value_get_pointer (value));
       break;
     case PROP_GROUPS:
-      priv->groups = (GSList *) g_value_get_pointer (value);
+      _set_groups (FROGR_ADD_TO_GROUP_DIALOG (object), g_value_get_pointer (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -455,8 +487,12 @@ _frogr_add_to_group_dialog_dispose (GObject *object)
       priv->pictures = NULL;
     }
 
-  if (priv->groups)
-    priv->groups = NULL;
+   if (priv->groups)
+    {
+      g_slist_foreach (priv->groups, (GFunc)g_object_unref, NULL);
+      g_slist_free (priv->groups);
+      priv->groups = NULL;
+    }
 
   if (priv->treemodel)
     {
@@ -511,10 +547,10 @@ frogr_add_to_group_dialog_init (FrogrAddToGroupDialog *self)
 
   /* Create widgets */
   gtk_dialog_add_buttons (GTK_DIALOG (self),
-                          GTK_STOCK_OK,
-                          GTK_RESPONSE_OK,
                           GTK_STOCK_CANCEL,
                           GTK_RESPONSE_CANCEL,
+                          GTK_STOCK_ADD,
+                          GTK_RESPONSE_ACCEPT,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
 
@@ -554,7 +590,9 @@ frogr_add_to_group_dialog_init (FrogrAddToGroupDialog *self)
 /* Public API */
 
 void
-frogr_add_to_group_dialog_show (GtkWindow *parent, GSList *pictures, GSList *groups)
+frogr_add_to_group_dialog_show (GtkWindow *parent,
+                                const GSList *pictures,
+                                const GSList *groups)
 {
   FrogrAddToGroupDialog *self = NULL;
   GObject *new = NULL;
