@@ -1,8 +1,8 @@
 /*
  * frogr-add-to-set-dialog.c -- 'Add to set' dialog
  *
- * Copyright (C) 2010-2011 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Copyright (C) 2010-2012 Mario Sanchez Prada
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -20,6 +20,8 @@
 
 #include "frogr-add-to-set-dialog.h"
 
+#include "frogr-controller.h"
+#include "frogr-model.h"
 #include "frogr-photoset.h"
 #include "frogr-picture.h"
 
@@ -68,6 +70,10 @@ enum {
 
 /* Prototypes */
 
+static void _set_pictures (FrogrAddToSetDialog *self, const GSList *pictures);
+
+static void _set_photosets (FrogrAddToSetDialog *self, const GSList *photosets);
+
 static GtkWidget *_create_tree_view (FrogrAddToSetDialog *self);
 
 static void _column_clicked_cb (GtkTreeViewColumn *col, gpointer data);
@@ -97,6 +103,24 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
 
 /* Private API */
 
+static void
+_set_pictures (FrogrAddToSetDialog *self, const GSList *pictures)
+{
+  FrogrAddToSetDialogPrivate *priv = FROGR_ADD_TO_SET_DIALOG_GET_PRIVATE (self);
+  priv->pictures = g_slist_copy ((GSList*) pictures);
+  g_slist_foreach (priv->pictures, (GFunc)g_object_ref, NULL);
+}
+
+static void
+_set_photosets (FrogrAddToSetDialog *self, const GSList *photosets)
+{
+  FrogrAddToSetDialogPrivate *priv = NULL;
+
+  priv = FROGR_ADD_TO_SET_DIALOG_GET_PRIVATE (self);
+  priv->photosets = g_slist_copy ((GSList*)photosets);
+  g_slist_foreach (priv->photosets, (GFunc)g_object_ref, NULL);
+}
+
 static GtkWidget *
 _create_tree_view (FrogrAddToSetDialog *self)
 {
@@ -376,10 +400,21 @@ _update_pictures (FrogrAddToSetDialog *self)
   priv = FROGR_ADD_TO_SET_DIALOG_GET_PRIVATE (self);
 
   selected_sets = _get_selected_photosets (self);
-  for (item = priv->pictures; item; item = g_slist_next (item))
+  if (selected_sets)
     {
-      picture = FROGR_PICTURE (item->data);
-      frogr_picture_set_photosets (picture, selected_sets);
+      FrogrModel *model = NULL;
+
+      for (item = priv->pictures; item; item = g_slist_next (item))
+        {
+          picture = FROGR_PICTURE (item->data);
+
+          /* frogr_picture_set_photosets expects transfer-full for photosets */
+          g_slist_foreach (selected_sets, (GFunc) g_object_ref, NULL);
+          frogr_picture_set_photosets (picture, g_slist_copy (selected_sets));
+        }
+
+      model = frogr_controller_get_model (frogr_controller_get_instance ());
+      frogr_model_notify_changes_in_pictures (model);
     }
   g_slist_free (selected_sets);
 }
@@ -387,7 +422,7 @@ _update_pictures (FrogrAddToSetDialog *self)
 static void
 _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
 {
-  if (response == GTK_RESPONSE_OK)
+  if (response == GTK_RESPONSE_ACCEPT)
     {
       FrogrAddToSetDialog *self = NULL;
 
@@ -405,15 +440,13 @@ _frogr_add_to_set_dialog_set_property (GObject *object,
                                        const GValue *value,
                                        GParamSpec *pspec)
 {
-  FrogrAddToSetDialogPrivate *priv = FROGR_ADD_TO_SET_DIALOG_GET_PRIVATE (object);
-
   switch (prop_id)
     {
     case PROP_PICTURES:
-      priv->pictures = (GSList *) g_value_get_pointer (value);
+      _set_pictures (FROGR_ADD_TO_SET_DIALOG (object), g_value_get_pointer (value));
       break;
     case PROP_PHOTOSETS:
-      priv->photosets = (GSList *) g_value_get_pointer (value);
+      _set_photosets (FROGR_ADD_TO_SET_DIALOG (object), g_value_get_pointer (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -455,8 +488,12 @@ _frogr_add_to_set_dialog_dispose (GObject *object)
       priv->pictures = NULL;
     }
 
-  if (priv->photosets)
-    priv->photosets = NULL;
+   if (priv->photosets)
+    {
+      g_slist_foreach (priv->photosets, (GFunc)g_object_unref, NULL);
+      g_slist_free (priv->photosets);
+      priv->photosets = NULL;
+    }
 
   if (priv->treemodel)
     {
@@ -511,10 +548,10 @@ frogr_add_to_set_dialog_init (FrogrAddToSetDialog *self)
 
   /* Create widgets */
   gtk_dialog_add_buttons (GTK_DIALOG (self),
-                          GTK_STOCK_OK,
-                          GTK_RESPONSE_OK,
                           GTK_STOCK_CANCEL,
                           GTK_RESPONSE_CANCEL,
+                          GTK_STOCK_ADD,
+                          GTK_RESPONSE_ACCEPT,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
 
@@ -554,7 +591,9 @@ frogr_add_to_set_dialog_init (FrogrAddToSetDialog *self)
 /* Public API */
 
 void
-frogr_add_to_set_dialog_show (GtkWindow *parent, GSList *pictures, GSList *photosets)
+frogr_add_to_set_dialog_show (GtkWindow *parent,
+                              const GSList *pictures,
+                              const GSList *photosets)
 {
   FrogrAddToSetDialog *self = NULL;
   GObject *new = NULL;
