@@ -2,7 +2,7 @@
  * frogr-auth-dialog.c -- Authorization dialog
  *
  * Copyright (C) 2009-2012 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -22,19 +22,11 @@
 
 #include "frogr-controller.h"
 #include "frogr-global-defs.h"
-#include "frogr-gtk-compat.h"
 #include "frogr-util.h"
 
 #include <config.h>
 #include <glib/gi18n.h>
 
-static gchar *unauth_txt =
-  N_("Please press the button to authorize %s "
-     "and then come back to complete the process.");
-
-static gchar *auth_txt =
-  N_("Enter verification code:");
-
 /* Prototypes */
 
 static void _ask_for_authorization (GtkWindow *parent);
@@ -57,7 +49,8 @@ _ask_for_authorization (GtkWindow *parent)
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_INFO,
                                    GTK_BUTTONS_OK,
-                                   _(unauth_txt),
+                                   _("Please press the button to authorize %s "
+                                     "and then come back to complete the process."),
                                    APP_SHORTNAME);
 
   title = g_strdup_printf (_("Authorize %s"), APP_SHORTNAME);
@@ -109,7 +102,8 @@ _build_verification_code_entry_widget (GtkWidget *dialog)
   gchar *entry_key = NULL;
   gint i = 0;
 
-  hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
   for (i = 0; i < 3; i++)
     {
       entry = gtk_entry_new ();
@@ -126,7 +120,7 @@ _build_verification_code_entry_widget (GtkWidget *dialog)
                     NULL);
       if (i < 2)
         {
-          separator = frogr_gtk_compat_separator_new (GTK_ORIENTATION_HORIZONTAL);
+          separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
           gtk_box_pack_start (GTK_BOX (hbox), separator, TRUE, TRUE, 0);
         }
     }
@@ -148,8 +142,8 @@ _ask_for_auth_confirmation (GtkWindow *parent)
   dialog = gtk_dialog_new_with_buttons (title,
                                         parent,
                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_OK,
-                                        GTK_RESPONSE_OK,
+                                        GTK_STOCK_CLOSE,
+                                        GTK_RESPONSE_CLOSE,
                                         NULL);
   g_free (title);
 
@@ -157,10 +151,11 @@ _ask_for_auth_confirmation (GtkWindow *parent)
 
   /* Fill action area */
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-  vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (vbox), FALSE);
 
   /* Description label */
-  label = gtk_label_new (auth_txt);
+  label = gtk_label_new (_("Enter verification code:"));
   gtk_misc_set_padding (GTK_MISC (label), 12, 0);
   gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 6);
 
@@ -191,7 +186,7 @@ _ask_for_auth_confirmation_response_cb (GtkDialog *dialog, gint response, gpoint
 {
   gboolean valid = FALSE;
 
-  if (response == GTK_RESPONSE_OK)
+  if (response == GTK_RESPONSE_CLOSE)
     {
       const gchar *vercode_part1 = NULL;
       const gchar *vercode_part2 = NULL;
