@@ -2,7 +2,7 @@
  * frogr-controller.h -- Controller of the whole application
  *
  * Copyright (C) 2009-2012 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -23,6 +23,7 @@
 
 #include "frogr-account.h"
 #include "frogr-main-view.h"
+#include "frogr-model.h"
 #include "frogr-picture.h"
 
 #include <config.h>
@@ -64,14 +65,13 @@ GType frogr_controller_get_type (void) G_GNUC_CONST;
 
 FrogrController *frogr_controller_get_instance (void);
 
+gint frogr_controller_run_app (FrogrController *self, int argc, char **argv);
+
 FrogrMainView *frogr_controller_get_main_view (FrogrController *self);
 
-gboolean frogr_controller_run_app (FrogrController *self);
+FrogrModel *frogr_controller_get_model (FrogrController *self);
 
-gboolean frogr_controller_quit_app (FrogrController *self);
-
-void frogr_controller_set_active_account (FrogrController *self,
-                                          FrogrAccount *account);
+void frogr_controller_set_active_account (FrogrController *self, const gchar *username);
 
 FrogrAccount *frogr_controller_get_active_account (FrogrController *self);
 
@@ -115,17 +115,21 @@ gboolean frogr_controller_is_authorized (FrogrController *self);
 
 void frogr_controller_revoke_authorization (FrogrController *self);
 
+gboolean frogr_controller_is_connected (FrogrController *self);
+
 void frogr_controller_load_pictures (FrogrController *self, GSList *fileuris);
 
-void frogr_controller_upload_pictures (FrogrController *self);
+void frogr_controller_upload_pictures (FrogrController *self, GSList *pictures);
 
 void frogr_controller_reorder_pictures (FrogrController *self);
 
-void frogr_controller_cancel_ongoing_request (FrogrController *self);
+void frogr_controller_cancel_ongoing_requests (FrogrController *self);
+
+gboolean frogr_controller_open_project_from_file (FrogrController *self, const gchar *path);
+
+gboolean frogr_controller_save_project_to_file (FrogrController *self, const gchar *path);
 
-#ifdef GTK_API_VERSION_3
 void frogr_controller_set_use_dark_theme (FrogrController *self, gboolean value);
-#endif
 
 G_END_DECLS
 
