@@ -1,8 +1,8 @@
 /*
  * frogr-create-new-set-dialog.c -- 'Create new set' dialog
  *
- * Copyright (C) 2010-2011 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Copyright (C) 2010-2012 Mario Sanchez Prada
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -22,8 +22,7 @@
 
 #include "frogr-photoset.h"
 #include "frogr-controller.h"
-#include "frogr-main-view-model.h"
-#include "frogr-main-view.h"
+#include "frogr-model.h"
 #include "frogr-picture.h"
 #include "frogr-util.h"
 
@@ -58,20 +57,42 @@ enum  {
 
 /* Prototypes */
 
+static void _set_pictures (FrogrCreateNewSetDialog *self, const GSList *pictures);
+
+static void _set_photosets (FrogrCreateNewSetDialog *self, const GSList *photosets);
+
 static void _on_button_toggled (GtkToggleButton *button, gpointer data);
 
 static gboolean _validate_dialog_data (FrogrCreateNewSetDialog *self);
 
 static gboolean _save_data (FrogrCreateNewSetDialog *self);
 
-static gboolean _update_model (FrogrCreateNewSetDialog *self,
-                               const gchar *title,
-                               const gchar *description);
+static void _update_model (FrogrCreateNewSetDialog *self,
+                           const gchar *title,
+                           const gchar *description);
 
 static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data);
 
 /* Private API */
 
+static void
+_set_pictures (FrogrCreateNewSetDialog *self, const GSList *pictures)
+{
+  FrogrCreateNewSetDialogPrivate *priv = FROGR_CREATE_NEW_SET_DIALOG_GET_PRIVATE (self);
+  priv->pictures = g_slist_copy ((GSList*) pictures);
+  g_slist_foreach (priv->pictures, (GFunc)g_object_ref, NULL);
+}
+
+static void
+_set_photosets (FrogrCreateNewSetDialog *self, const GSList *photosets)
+{
+  FrogrCreateNewSetDialogPrivate *priv = NULL;
+
+  priv = FROGR_CREATE_NEW_SET_DIALOG_GET_PRIVATE (self);
+  priv->photosets = g_slist_copy ((GSList*)photosets);
+  g_slist_foreach (priv->photosets, (GFunc)g_object_ref, NULL);
+}
+
 static void
 _on_button_toggled (GtkToggleButton *button, gpointer data)
 {
@@ -127,9 +148,10 @@ _save_data (FrogrCreateNewSetDialog *self)
                                           &start, &end, FALSE);
   description = g_strstrip (description);
 
-  /* validate dialog */
-  if (_validate_dialog_data (self))
-    result = _update_model (self, title, description);
+  /* validate and update */
+  result = _validate_dialog_data (self);
+  if (result)
+    _update_model (self, title, description);
   else
     frogr_util_show_error_dialog (GTK_WINDOW (self), _("Missing data required"));
 
@@ -141,28 +163,25 @@ _save_data (FrogrCreateNewSetDialog *self)
   return result;
 }
 
-static gboolean
+static void
 _update_model (FrogrCreateNewSetDialog *self,
                const gchar *title,
                const gchar *description)
 {
   FrogrCreateNewSetDialogPrivate *priv = NULL;
   FrogrController *controller = NULL;
-  FrogrMainView *mainview = NULL;
-  FrogrMainViewModel *mainview_model = NULL;
+  FrogrModel *model = NULL;
   FrogrPhotoSet *new_set = NULL;
   FrogrPicture *picture = NULL;
   GSList *item = NULL;
-  gboolean result = FALSE;
 
   priv = FROGR_CREATE_NEW_SET_DIALOG_GET_PRIVATE (self);
   controller = frogr_controller_get_instance ();
-  mainview = frogr_controller_get_main_view (controller);
-  mainview_model = frogr_main_view_get_model (mainview);
+  model = frogr_controller_get_model (controller);
 
   /* Add the set to the model */
-  new_set = frogr_photoset_new (title, description);
-  frogr_main_view_model_add_photoset (mainview_model, new_set);
+  new_set = frogr_photoset_new_local (title, description);
+  frogr_model_add_local_photoset (model, new_set);
 
   /* Add the set to the list of sets for each picture */
   for (item = priv->pictures; item; item = g_slist_next (item))
@@ -176,11 +195,10 @@ _update_model (FrogrCreateNewSetDialog *self,
           frogr_picture_set_title (picture, title);
           frogr_picture_set_description (picture, description);
         }
-
-      result = TRUE;
     }
+  g_object_unref (new_set);
 
-  return result;
+  frogr_model_notify_changes_in_pictures (model);
 }
 
 static void
@@ -190,7 +208,7 @@ _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
 
   /* Try to save data if response is OK */
   self = FROGR_CREATE_NEW_SET_DIALOG (dialog);
-  if (response == GTK_RESPONSE_OK && _save_data (self) == FALSE)
+  if (response == GTK_RESPONSE_ACCEPT && _save_data (self) == FALSE)
     return;
 
   /* Destroy the dialog */
@@ -203,15 +221,13 @@ _frogr_create_new_set_dialog_set_property (GObject *object,
                                            const GValue *value,
                                            GParamSpec *pspec)
 {
-  FrogrCreateNewSetDialogPrivate *priv = FROGR_CREATE_NEW_SET_DIALOG_GET_PRIVATE (object);
-
   switch (prop_id)
     {
     case PROP_PICTURES:
-      priv->pictures = (GSList *) g_value_get_pointer (value);
+      _set_pictures (FROGR_CREATE_NEW_SET_DIALOG (object), g_value_get_pointer (value));
       break;
     case PROP_PHOTOSETS:
-      priv->photosets = (GSList *) g_value_get_pointer (value);
+      _set_photosets (FROGR_CREATE_NEW_SET_DIALOG (object), g_value_get_pointer (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -254,7 +270,11 @@ _frogr_create_new_set_dialog_dispose (GObject *object)
     }
 
   if (priv->photosets)
-    priv->photosets = NULL;
+    {
+      g_slist_foreach (priv->photosets, (GFunc)g_object_unref, NULL);
+      g_slist_free (priv->photosets);
+      priv->photosets = NULL;
+    }
 
   G_OBJECT_CLASS(frogr_create_new_set_dialog_parent_class)->dispose (object);
 }
@@ -295,8 +315,7 @@ frogr_create_new_set_dialog_init (FrogrCreateNewSetDialog *self)
 {
   FrogrCreateNewSetDialogPrivate *priv = NULL;
   GtkWidget *vbox = NULL;
-  GtkWidget *table = NULL;
-  GtkWidget *align = NULL;
+  GtkWidget *grid = NULL;
   GtkWidget *scroller = NULL;
   GtkWidget *widget = NULL;
 
@@ -306,36 +325,33 @@ frogr_create_new_set_dialog_init (FrogrCreateNewSetDialog *self)
 
   /* Create widgets */
   gtk_dialog_add_buttons (GTK_DIALOG (self),
-                          GTK_STOCK_OK,
-                          GTK_RESPONSE_OK,
                           GTK_STOCK_CANCEL,
                           GTK_RESPONSE_CANCEL,
+                          GTK_STOCK_ADD,
+                          GTK_RESPONSE_ACCEPT,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
 
   vbox = gtk_dialog_get_content_area (GTK_DIALOG (self));
 
-  table = gtk_table_new (2, 2, FALSE);
-  gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 6);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_box_pack_start (GTK_BOX (vbox), grid, TRUE, TRUE, 6);
 
   widget = gtk_label_new (_("Title:"));
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), widget);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 0, 1,
-                    0, 0, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (widget), GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), widget, 0, 0, 1, 1);
 
   widget = gtk_entry_new ();
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), widget);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 0, 1,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (widget), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), widget, 1, 0, 1, 1);
   priv->title_entry = widget;
 
   widget = gtk_label_new (_("Description:"));
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), widget);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 1, 2,
-                    0, GTK_EXPAND | GTK_FILL, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (widget), GTK_ALIGN_END);
+  gtk_widget_set_valign (GTK_WIDGET (widget), GTK_ALIGN_START);
+  gtk_grid_attach (GTK_GRID (grid), widget, 0, 1, 1, 1);
 
   widget = gtk_text_view_new ();
   scroller = gtk_scrolled_window_new (NULL, NULL);
@@ -346,19 +362,16 @@ frogr_create_new_set_dialog_init (FrogrCreateNewSetDialog *self)
   gtk_container_add (GTK_CONTAINER (scroller), widget);
   gtk_text_view_set_accepts_tab (GTK_TEXT_VIEW (widget), FALSE);
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (widget), GTK_WRAP_WORD);
-  align = gtk_alignment_new (1, 0, 1, 1);
-  gtk_container_add (GTK_CONTAINER (align), scroller);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 1, 2,
-                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (scroller), TRUE);
+  gtk_widget_set_vexpand (GTK_WIDGET (scroller), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), scroller, 1, 1, 1, 1);
 
   priv->description_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
   priv->description_tv = widget;
 
   widget = gtk_check_button_new_with_mnemonic (_("Fill Pictures Details with Title and Description"));
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), widget);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 2, 3,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (widget), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), widget, 1, 2, 1, 1);
   priv->copy_to_pictures_cb = widget;
 
   g_signal_connect (G_OBJECT (priv->copy_to_pictures_cb), "toggled",
@@ -371,7 +384,9 @@ frogr_create_new_set_dialog_init (FrogrCreateNewSetDialog *self)
 /* Public API */
 
 void
-frogr_create_new_set_dialog_show (GtkWindow *parent, GSList *pictures, GSList *photosets)
+frogr_create_new_set_dialog_show (GtkWindow *parent,
+                                  const GSList *pictures,
+                                  const GSList *photosets)
 {
   GtkWidget *dialog = NULL;
   dialog = GTK_WIDGET (g_object_new (FROGR_TYPE_CREATE_NEW_SET_DIALOG,
