@@ -2,7 +2,7 @@
  * frogr-details-dialog.c -- Picture details dialog
  *
  * Copyright (C) 2009-2012 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -23,10 +23,9 @@
 #include "frogr-config.h"
 #include "frogr-controller.h"
 #include "frogr-global-defs.h"
-#include "frogr-gtk-compat.h"
 #include "frogr-live-entry.h"
 #include "frogr-main-view.h"
-#include "frogr-main-view-model.h"
+#include "frogr-model.h"
 #include "frogr-picture.h"
 #include "frogr-util.h"
 
@@ -88,7 +87,7 @@ enum  {
 
 
 static const gchar *license_descriptions[] = {
-  N_("Default (as specified in flickr)"),
+  N_("Default (as specified in Flickr)"),
   N_("All rights reserved"),
   N_("CC Attribution-NonCommercial-ShareAlike"),
   N_("CC Attribution-NonCommercial"),
@@ -102,6 +101,8 @@ static const gchar *license_descriptions[] = {
 
 /* Prototypes */
 
+static void _set_pictures (FrogrDetailsDialog *self, const GSList *pictures);
+
 static void _create_widgets (FrogrDetailsDialog *self);
 
 static void _update_ui (FrogrDetailsDialog *self);
@@ -131,6 +132,14 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
 
 /* Private API */
 
+static void
+_set_pictures (FrogrDetailsDialog *self, const GSList *pictures)
+{
+  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
+  priv->pictures = g_slist_copy ((GSList*) pictures);
+  g_slist_foreach (priv->pictures, (GFunc)g_object_ref, NULL);
+}
+
 static void
 _create_widgets (FrogrDetailsDialog *self)
 {
@@ -144,11 +153,10 @@ _create_widgets (FrogrDetailsDialog *self)
   GtkWidget *private_vbox = NULL;
   GtkWidget *content_type_hbox = NULL;
   GtkWidget *safety_level_hbox = NULL;
-
   GtkWidget *align = NULL;
   GtkWidget *label = NULL;
   GtkWidget *widget = NULL;
-  GtkWidget *table = NULL;
+  GtkWidget *grid = NULL;
   GtkWidget *scroller = NULL;
   gchar *markup = NULL;
   gint i;
@@ -157,8 +165,10 @@ _create_widgets (FrogrDetailsDialog *self)
 
   main_vbox = gtk_dialog_get_content_area (GTK_DIALOG (self));
 
-  hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (vbox), FALSE);
 
   /* Left side (image, radio buttons, checkboxes...) */
 
@@ -182,8 +192,10 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Visibility */
 
-  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  visibility_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (section_vbox), FALSE);
+  visibility_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (visibility_vbox), FALSE);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Visibility"));
@@ -195,7 +207,8 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
 
-  internal_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (internal_hbox), FALSE);
 
   widget = gtk_radio_button_new_with_mnemonic (NULL, _("_Private"));
   gtk_box_pack_start (GTK_BOX (internal_hbox), widget, FALSE, FALSE, 0);
@@ -207,7 +220,8 @@ _create_widgets (FrogrDetailsDialog *self)
 
   gtk_box_pack_start (GTK_BOX (visibility_vbox), internal_hbox, FALSE, FALSE, 0);
 
-  private_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  private_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (private_vbox), FALSE);
 
   widget = gtk_check_button_new_with_mnemonic (_("Visible to _Family"));
   gtk_box_pack_start (GTK_BOX (private_vbox), widget, FALSE, FALSE, 0);
@@ -217,12 +231,14 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_box_pack_start (GTK_BOX (private_vbox), widget, FALSE, FALSE, 0);
   priv->friend_cb = widget;
 
-  internal_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (internal_hbox), FALSE);
 
   gtk_box_pack_start (GTK_BOX (internal_hbox), private_vbox, FALSE, FALSE, 12);
   gtk_box_pack_start (GTK_BOX (visibility_vbox), internal_hbox, FALSE, FALSE, 0);
 
-  internal_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (internal_hbox), FALSE);
 
   gtk_box_pack_start (GTK_BOX (internal_hbox), visibility_vbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (section_vbox), internal_hbox, FALSE, FALSE, 0);
@@ -231,8 +247,10 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Content type */
 
-  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  content_type_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (section_vbox), FALSE);
+  content_type_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (content_type_hbox), FALSE);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Content Type"));
@@ -262,8 +280,10 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Safety level */
 
-  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  safety_level_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (section_vbox), FALSE);
+  safety_level_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (safety_level_hbox), FALSE);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Safety Level"));
@@ -293,7 +313,8 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* License type */
 
-  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (section_vbox), FALSE);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("License Type"));
@@ -305,9 +326,9 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
 
-  widget = frogr_gtk_compat_combo_box_text_new ();
+  widget = gtk_combo_box_text_new ();
   for (i = 0; license_descriptions[i]; i++)
-    frogr_gtk_compat_combo_box_text_insert (GTK_COMBO_BOX (widget), i, _(license_descriptions[i]));
+    gtk_combo_box_text_insert (GTK_COMBO_BOX_TEXT (widget), i, NULL, _(license_descriptions[i]));
 
   priv->license_cb = widget;
   gtk_box_pack_start (GTK_BOX (section_vbox), widget, FALSE, FALSE, 0);
@@ -316,7 +337,8 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Other properties */
 
-  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (section_vbox), FALSE);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Other Properties"));
@@ -342,25 +364,24 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Right side (text fields) */
 
-  table = gtk_table_new (3, 2, FALSE);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
 
   label = gtk_label_new_with_mnemonic (_("_Title:"));
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), label);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 0, 1,
-                    0, 0, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (label), GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
 
   widget = gtk_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
-  gtk_table_attach (GTK_TABLE (table), widget, 1, 2, 0, 1,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (widget), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), widget, 1, 0, 1, 1);
   priv->title_entry = widget;
 
   label = gtk_label_new_with_mnemonic (_("_Description:"));
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), label);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 1, 2,
-                    0, GTK_EXPAND | GTK_FILL, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (label), GTK_ALIGN_END);
+  gtk_widget_set_valign (GTK_WIDGET (label), GTK_ALIGN_START);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
 
   widget = gtk_text_view_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
@@ -374,27 +395,24 @@ _create_widgets (FrogrDetailsDialog *self)
                                        GTK_SHADOW_ETCHED_IN);
   gtk_container_add (GTK_CONTAINER (scroller), widget);
 
-  align = gtk_alignment_new (1, 0, 1, 1);
-  gtk_container_add (GTK_CONTAINER (align), scroller);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 1, 2,
-                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (scroller), TRUE);
+  gtk_widget_set_vexpand (GTK_WIDGET (scroller), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), scroller, 1, 1, 1, 1);
   priv->desc_tview = widget;
   priv->text_buffer =
     gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->desc_tview));
 
   label = gtk_label_new_with_mnemonic (_("Ta_gs:"));
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), label);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 2, 3,
-                    0, 0, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (label), GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
 
   widget = frogr_live_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
-  gtk_table_attach (GTK_TABLE (table), widget, 1, 2, 2, 3,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (widget), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), widget, 1, 2, 1, 1);
   priv->tags_entry = widget;
 
-  gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), grid, TRUE, TRUE, 0);
 
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, TRUE, TRUE, 6);
 
@@ -516,30 +534,26 @@ _load_picture_from_disk_cb (GObject *object,
   file = G_FILE (object);
   if (g_file_load_contents_finish (file, res, &contents, &length, NULL, &error))
     {
-      GdkPixbufLoader *pixbuf_loader = gdk_pixbuf_loader_new ();
+      FrogrPicture *picture = NULL;
+      GdkPixbuf *pixbuf = NULL;
 
-      if (gdk_pixbuf_loader_write (pixbuf_loader,
-                                   (const guchar *)contents,
-                                   length,
-                                   &error))
+      picture = FROGR_PICTURE (priv->pictures->data);
+      if (frogr_picture_is_video (picture))
+        pixbuf = frogr_util_get_pixbuf_for_video_file (file, PICTURE_WIDTH, PICTURE_HEIGHT, &error);
+      else
+        pixbuf = frogr_util_get_pixbuf_from_image_contents ((const guchar *)contents, length,
+                                                            PICTURE_WIDTH, PICTURE_HEIGHT, &error);
+
+      if (pixbuf)
         {
-          GdkPixbuf *pixbuf = NULL;
-          GdkPixbuf *s_pixbuf = NULL;
-
-          gdk_pixbuf_loader_close (pixbuf_loader, NULL);
-          pixbuf = gdk_pixbuf_loader_get_pixbuf (pixbuf_loader);
-
-          /* Get (scaled, and maybe rotated) pixbuf */
-          s_pixbuf = frogr_util_get_corrected_pixbuf (pixbuf, PICTURE_WIDTH, PICTURE_HEIGHT);
-
-          gtk_image_set_from_pixbuf (GTK_IMAGE (priv->picture_img), s_pixbuf);
-          g_object_unref (s_pixbuf);
-
-          /* Everything should be fine by now, show it */
-          _place_picture_in_dialog_and_show (self);
+          gtk_image_set_from_pixbuf (GTK_IMAGE (priv->picture_img), pixbuf);
+          g_object_unref (pixbuf);
         }
 
-      g_object_unref (pixbuf_loader);
+      /* Everything should be fine by now, show it */
+      _place_picture_in_dialog_and_show (self);
+
+      g_free (contents);
     }
 
   /* Show error to the user and finalize dialog if needed */
@@ -943,10 +957,13 @@ _save_data (FrogrDetailsDialog *self)
   /* validate dialog */
   if (_validate_dialog_data (self))
     {
+      FrogrModel *model = NULL;
       FrogrPicture *picture;
       GSList *item;
       guint n_pictures;
 
+      model = frogr_controller_get_model (frogr_controller_get_instance ());
+
       /* Iterate over the rest of elements */
       n_pictures = g_slist_length (priv->pictures);
       for (item = priv->pictures; item; item = g_slist_next (item))
@@ -992,14 +1009,10 @@ _save_data (FrogrDetailsDialog *self)
 
       /* Add tags to the model */
       if (!g_str_equal (tags, ""))
-        {
-          FrogrMainView *mainview = NULL;
-          FrogrMainViewModel *model = NULL;
+        frogr_model_add_local_tags_from_string (model, tags);
 
-          mainview = frogr_controller_get_main_view (frogr_controller_get_instance ());
-          model = frogr_main_view_get_model (mainview);
-          frogr_main_view_model_add_local_tags_from_string (model, tags);
-        }
+      /* Notify the model that pictures details have probably changed */
+      frogr_model_notify_changes_in_pictures (model);
     }
   else
     {
@@ -1053,25 +1066,8 @@ _on_toggle_button_toggled (GtkToggleButton *tbutton, gpointer data)
 static void
 _on_picture_button_clicked (GtkButton *button, gpointer data)
 {
-  FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (data);
-  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
-  GSList *current_pic = NULL;
-  GList *uris_list = NULL;
-  FrogrPicture *picture = NULL;
-  gchar *fileuri = NULL;
-
-  for (current_pic = priv->pictures; current_pic; current_pic = g_slist_next (current_pic))
-    {
-      picture = FROGR_PICTURE (current_pic->data);
-      fileuri = g_strdup (frogr_picture_get_fileuri (picture));
-
-      /* Dupped uris in the GList must NOT be freed here */
-      uris_list = g_list_append (uris_list, fileuri);
-    }
-
-  frogr_util_open_images_in_viewer (uris_list);
-
-  g_list_free (uris_list);
+  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (data);
+  frogr_util_open_pictures_in_viewer (priv->pictures);
 }
 
 static void
@@ -1080,13 +1076,13 @@ _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
   FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (dialog);
 
   /* Try to save data if response is OK */
-  if (response == GTK_RESPONSE_OK && _save_data (self) == FALSE)
+  if (response == GTK_RESPONSE_ACCEPT && _save_data (self) == FALSE)
     return;
 
   gtk_widget_destroy (GTK_WIDGET (self));
 
   /* Ensure pictures are reordered on OK */
-  if (response == GTK_RESPONSE_OK)
+  if (response == GTK_RESPONSE_ACCEPT)
     frogr_controller_reorder_pictures (frogr_controller_get_instance ());
 }
 
@@ -1096,12 +1092,10 @@ _frogr_details_dialog_set_property (GObject *object,
                                     const GValue *value,
                                     GParamSpec *pspec)
 {
-  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (object);
-
   switch (prop_id)
     {
     case PROP_PICTURES:
-      priv->pictures = (GSList *) g_value_get_pointer (value);
+      _set_pictures (FROGR_DETAILS_DIALOG (object), g_value_get_pointer (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1197,7 +1191,7 @@ frogr_details_dialog_init (FrogrDetailsDialog *self)
 
   gtk_dialog_add_buttons (GTK_DIALOG (self),
                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                          GTK_STOCK_OK, GTK_RESPONSE_OK,
+                          GTK_STOCK_EDIT, GTK_RESPONSE_ACCEPT,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
 
@@ -1205,21 +1199,23 @@ frogr_details_dialog_init (FrogrDetailsDialog *self)
                     G_CALLBACK (_dialog_response_cb), NULL);
 
   gtk_dialog_set_default_response (GTK_DIALOG (self),
-                                   GTK_RESPONSE_OK);
+                                   GTK_RESPONSE_ACCEPT);
 }
 
 
 /* Public API */
 
 void
-frogr_details_dialog_show (GtkWindow *parent, GSList *fpictures, GSList *tags)
+frogr_details_dialog_show (GtkWindow *parent,
+                           const GSList *pictures,
+                           const GSList *tags)
 {
   FrogrConfig *config = NULL;
   GObject *new = NULL;
 
   new = g_object_new (FROGR_TYPE_DETAILS_DIALOG,
                       "modal", TRUE,
-                      "pictures", fpictures,
+                      "pictures", pictures,
                       "transient-for", parent,
                       "width-request", DIALOG_MIN_WIDTH,
                       "height-request", -1,
