@@ -2,7 +2,7 @@
  * frogr-settings-dialog.c -- Main settings dialog
  *
  * Copyright (C) 2009-2012 Mario Sanchez Prada
- * Authors: Mario Sanchez Prada <msanchez@igalia.com>
+ * Authors: Mario Sanchez Prada <msanchez@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 3 of the GNU General Public
@@ -23,7 +23,6 @@
 #include "frogr-config.h"
 #include "frogr-controller.h"
 #include "frogr-global-defs.h"
-#include "frogr-gtk-compat.h"
 #include "frogr-util.h"
 
 #include <config.h>
@@ -69,9 +68,7 @@ typedef struct _FrogrSettingsDialogPrivate {
   GtkWidget *enable_tags_autocompletion_cb;
   GtkWidget *keep_file_extensions_cb;
   GtkWidget *import_tags_cb;
-#ifdef GTK_API_VERSION_3
   GtkWidget *use_dark_theme_cb;
-#endif
 
   gboolean public_visibility;
   gboolean family_visibility;
@@ -81,9 +78,7 @@ typedef struct _FrogrSettingsDialogPrivate {
   gboolean enable_tags_autocompletion;
   gboolean keep_file_extensions;
   gboolean import_tags;
-#ifdef GTK_API_VERSION_3
   gboolean use_dark_theme;
-#endif
 
   FspLicense license;
   FspSafetyLevel safety_level;
@@ -102,7 +97,7 @@ typedef struct _FrogrSettingsDialogPrivate {
 static FrogrSettingsDialog *_instance = NULL;
 
 static const gchar *license_descriptions[] = {
-  N_("Default (as specified in flickr)"),
+  N_("Default (as specified in Flickr)"),
   N_("All rights reserved"),
   N_("CC Attribution-NonCommercial-ShareAlike"),
   N_("CC Attribution-NonCommercial"),
@@ -183,7 +178,8 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
 
-  vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (vbox), FALSE);
 
   /* Default Visibility */
 
@@ -198,8 +194,10 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  box1 = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  box2 = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  box1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (box1), FALSE);
+  box2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_box_set_homogeneous (GTK_BOX (box2), FALSE);
 
   _add_toggleable_item (self, GTK_BOX (box2), NULL, TRUE, _("_Private"), &priv->private_rb);
   _add_toggleable_item (self, GTK_BOX (box2), GTK_RADIO_BUTTON (priv->private_rb),
@@ -207,14 +205,16 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
 
-  box2 = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  box2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (box2), FALSE);
 
   _add_toggleable_item (self, GTK_BOX (box2), NULL, FALSE,
                         _("Visible to _Family"), &priv->family_cb);
   _add_toggleable_item (self, GTK_BOX (box2), NULL, FALSE,
                         _("Visible to F_riends"), &priv->friend_cb);
 
-  padding_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  padding_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_set_homogeneous (GTK_BOX (padding_hbox), FALSE);
 
   gtk_box_pack_start (GTK_BOX (padding_hbox), box2, FALSE, FALSE, 12);
   gtk_box_pack_start (GTK_BOX (box1), padding_hbox, FALSE, FALSE, 0);
@@ -234,7 +234,8 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  box1 = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  box1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_box_set_homogeneous (GTK_BOX (box1), FALSE);
 
   _add_toggleable_item (self, GTK_BOX (box1), NULL, TRUE,
                         _("P_hoto"), &priv->photo_content_rb);
@@ -258,7 +259,8 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  box1 = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  box1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_box_set_homogeneous (GTK_BOX (box1), FALSE);
 
   _add_toggleable_item (self, GTK_BOX (box1), NULL, TRUE,
                         _("S_afe"), &priv->safe_rb);
@@ -282,9 +284,9 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  combo = frogr_gtk_compat_combo_box_text_new ();
+  combo = gtk_combo_box_text_new ();
   for (i = 0; license_descriptions[i]; i++)
-    frogr_gtk_compat_combo_box_text_insert (GTK_COMBO_BOX (combo), i, _(license_descriptions[i]));
+    gtk_combo_box_text_insert (GTK_COMBO_BOX_TEXT (combo), i, NULL, _(license_descriptions[i]));
 
   gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 0);
   priv->license_cb = combo;
@@ -306,7 +308,8 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  box1 = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  box1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (box1), FALSE);
 
   _add_toggleable_item (self, GTK_BOX (box1), NULL, FALSE,
                         _("Set Geo_location Information for Pictures"),
@@ -326,7 +329,7 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 {
   FrogrSettingsDialogPrivate *priv = NULL;
   GtkWidget *vbox = NULL;
-  GtkWidget *table = NULL;
+  GtkWidget *grid = NULL;
   GtkWidget *align = NULL;
   GtkWidget *cbutton = NULL;
   GtkWidget *label = NULL;
@@ -335,7 +338,8 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
 
-  vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (vbox), FALSE);
 
   /* Proxy settings */
 
@@ -360,77 +364,59 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   /* Proxy host */
 
-  table = gtk_table_new (2, 4, FALSE);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
 
   label = gtk_label_new_with_mnemonic (_("_Host:"));
-  align = gtk_alignment_new (0, 0, 0, 0);
-  gtk_container_add (GTK_CONTAINER (align), label);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 0, 1,
-                    GTK_FILL, 0, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (label), GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
   priv->proxy_host_label = label;
 
   entry = gtk_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), entry);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 0, 1,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (entry), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), entry, 1, 0, 1, 1);
   priv->proxy_host_entry = entry;
 
   /* Proxy port */
 
   label = gtk_label_new_with_mnemonic (_("_Port:"));
-  align = gtk_alignment_new (0, 0, 0, 0);
-  gtk_container_add (GTK_CONTAINER (align), label);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 1, 2,
-                    GTK_FILL, 0, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (label), GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
   priv->proxy_port_label = label;
 
   entry = gtk_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), entry);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 1, 2,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (entry), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), entry, 1, 1, 1, 1);
   priv->proxy_port_entry = entry;
 
   /* Proxy username */
 
   label = gtk_label_new_with_mnemonic (_("U_sername:"));
-  align = gtk_alignment_new (0, 0, 0, 0);
-  gtk_container_add (GTK_CONTAINER (align), label);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 2, 3,
-                    GTK_FILL, 0, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (label), GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
   priv->proxy_username_label = label;
 
   entry = gtk_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), entry);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 2, 3,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (entry), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), entry, 1, 2, 1, 1);
   priv->proxy_username_entry = entry;
 
   /* Proxy password */
 
   label = gtk_label_new_with_mnemonic (_("Pass_word:"));
-  align = gtk_alignment_new (0, 0, 0, 0);
-  gtk_container_add (GTK_CONTAINER (align), label);
-  gtk_table_attach (GTK_TABLE (table), align, 0, 1, 3, 4,
-                    GTK_FILL, 0, 6, 6);
+  gtk_widget_set_halign (GTK_WIDGET (label), GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 3, 1, 1);
   priv->proxy_password_label = label;
 
   entry = gtk_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
-
   gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), entry);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 3, 4,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (entry), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), entry, 1, 3, 1, 1);
   priv->proxy_password_entry = entry;
 
 #ifdef HAVE_LIBSOUP_GNOME
@@ -438,14 +424,12 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   /* Use GNOME General Proxy Settings */
 
   cbutton = gtk_check_button_new_with_mnemonic (_("_Use GNOME General Proxy Settings"));
-  align = gtk_alignment_new (1, 0, 1, 0);
-  gtk_container_add (GTK_CONTAINER (align), cbutton);
-  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 4, 5,
-                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  gtk_widget_set_hexpand (GTK_WIDGET (cbutton), TRUE);
+  gtk_grid_attach (GTK_GRID (grid), cbutton, 1, 4, 1, 1);
   priv->use_gnome_proxy_cb = cbutton;
 #endif
 
-  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), grid, FALSE, FALSE, 0);
 
   /* Connect signals */
   g_signal_connect (G_OBJECT (priv->use_proxy_cb), "toggled",
@@ -478,7 +462,8 @@ _add_misc_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
 
-  vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (vbox), FALSE);
 
   /* Other Stuff */
 
@@ -493,7 +478,8 @@ _add_misc_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  box = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_set_homogeneous (GTK_BOX (box), FALSE);
 
   _add_toggleable_item (self, GTK_BOX (box), NULL, FALSE,
                         _("Ena_ble Tags Auto-Completion"),
@@ -502,11 +488,9 @@ _add_misc_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
                         _("_Import Tags from Pictures Metadata"),
                         &priv->import_tags_cb);
 
-#ifdef GTK_API_VERSION_3
   _add_toggleable_item (self, GTK_BOX (box), NULL, FALSE,
                         _("Use _Dark GTK Theme"),
                         &priv->use_dark_theme_cb);
-#endif
 
   _add_toggleable_item (self, GTK_BOX (box), NULL, FALSE,
                         _("_Keep File Extensions in Titles when Loading"),
@@ -536,9 +520,7 @@ _fill_dialog_with_data (FrogrSettingsDialog *self)
   priv->enable_tags_autocompletion = frogr_config_get_tags_autocompletion (priv->config);
   priv->keep_file_extensions = frogr_config_get_keep_file_extensions (priv->config);
   priv->import_tags = frogr_config_get_import_tags_from_metadata (priv->config);
-#ifdef GTK_API_VERSION_3
   priv->use_dark_theme = frogr_config_get_use_dark_theme (priv->config);
-#endif
   priv->use_proxy = frogr_config_get_use_proxy (priv->config);
 #ifdef HAVE_LIBSOUP_GNOME
   priv->use_gnome_proxy = frogr_config_get_use_gnome_proxy (priv->config);
@@ -604,10 +586,8 @@ _fill_dialog_with_data (FrogrSettingsDialog *self)
                                 priv->keep_file_extensions);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->import_tags_cb),
                                 priv->import_tags);
-#ifdef GTK_API_VERSION_3
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->use_dark_theme_cb),
                                 priv->use_dark_theme);
-#endif
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->use_proxy_cb),
                                 priv->use_proxy);
 #ifdef HAVE_LIBSOUP_GNOME
@@ -651,9 +631,7 @@ _save_data (FrogrSettingsDialog *self)
   frogr_config_set_tags_autocompletion (priv->config, priv->enable_tags_autocompletion);
   frogr_config_set_keep_file_extensions (priv->config, priv->keep_file_extensions);
   frogr_config_set_import_tags_from_metadata (priv->config, priv->import_tags);
-#ifdef GTK_API_VERSION_3
   frogr_config_set_use_dark_theme (priv->config, priv->use_dark_theme);
-#endif
 
   frogr_config_set_use_proxy (priv->config, priv->use_proxy);
 #ifdef HAVE_LIBSOUP_GNOME
@@ -816,13 +794,11 @@ _on_button_toggled (GtkToggleButton *button, gpointer data)
       DEBUG ("import tags from pictures metadata set to %s", active ? "TRUE" : "FALSE");
     }
 
-#ifdef GTK_API_VERSION_3
   if (GTK_WIDGET (button) == priv->use_dark_theme_cb)
     {
       priv->use_dark_theme = active;
       DEBUG ("Use Dark Theme set to %s", active ? "TRUE" : "FALSE");
     }
-#endif
 
   if (GTK_WIDGET (button) == priv->use_proxy_cb)
     {
@@ -888,7 +864,7 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
   FrogrSettingsDialog *self = FROGR_SETTINGS_DIALOG (dialog);
   FrogrSettingsDialogPrivate *priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
 
-  if (response == GTK_RESPONSE_OK)
+  if (response == GTK_RESPONSE_CLOSE)
     {
       /* Try to save data if response is OK */
       if (!_save_data (self))
@@ -907,10 +883,8 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
       else
         frogr_controller_set_proxy (priv->controller, FALSE, NULL, NULL, NULL, NULL);
 
-#ifdef GTK_API_VERSION_3
       /* Update dark theme related stuff */
       frogr_controller_set_use_dark_theme (priv->controller, priv->use_dark_theme);
-#endif
     }
 
   gtk_widget_hide (GTK_WIDGET (self));
@@ -986,9 +960,7 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
   priv->enable_tags_autocompletion_cb = NULL;
   priv->keep_file_extensions_cb = NULL;
   priv->import_tags_cb = NULL;
-#ifdef GTK_API_VERSION_3
   priv->use_dark_theme_cb = NULL;
-#endif
   priv->use_proxy_cb = NULL;
   priv->use_gnome_proxy_cb = NULL;
   priv->proxy_host_label = NULL;
@@ -1010,9 +982,7 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
   priv->enable_tags_autocompletion = TRUE;
   priv->keep_file_extensions = FALSE;
   priv->import_tags = TRUE;
-#ifdef GTK_API_VERSION_3
   priv->use_dark_theme = TRUE;
-#endif
   priv->use_proxy = FALSE;
   priv->use_gnome_proxy = FALSE;
   priv->proxy_host = NULL;
@@ -1022,10 +992,10 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
 
   /* Create widgets */
   gtk_dialog_add_buttons (GTK_DIALOG (self),
-                          GTK_STOCK_OK,
-                          GTK_RESPONSE_OK,
                           GTK_STOCK_CANCEL,
                           GTK_RESPONSE_CANCEL,
+                          GTK_STOCK_CLOSE,
+                          GTK_RESPONSE_CLOSE,
                           NULL);
   gtk_container_set_border_width (GTK_CONTAINER (self), 6);
 
@@ -1048,7 +1018,7 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
                     NULL);
 
   gtk_dialog_set_default_response (GTK_DIALOG (self),
-                                   GTK_RESPONSE_OK);
+                                   GTK_RESPONSE_CLOSE);
 }
 
 
